package com.yihu.wlyy.web.patient.sign;

import com.yihu.wlyy.entity.patient.SignWeb;
import com.yihu.wlyy.service.app.sign.SignWebService;
import com.yihu.wlyy.web.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

@RestController
@RequestMapping(value = "/patient/sign", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
@Api(description = "患者端-签约管理")
public class PatientSignController extends BaseController {

    @Autowired
    private SignWebService signWebService;

    /**
     * 个人医生签约
     *
     * @param doctor    医生标识
     * @param startTime 签约开始时间
     * @param endTime   到期时间
     * @return 申请结果
     */
    @RequestMapping("/sign_doctor")
    public String signDoctor(@RequestParam(value = "doctor") String doctor,
                             @RequestParam(value = "startTime", required = false) String startTime,
                             @RequestParam(value = "endTime", required = false) String endTime) {
        try {
            List<Object> listDoctor = signWebService.getSignInfoByStatus(getRepUID());
//			List<Object> listDoctor = signWebService.getSignInfoByStatus(getUID());
            if (listDoctor.size() > 0) {
                return error(-1, "存在已签约的医生,请确认!");
            }

            Page<Object> list = signWebService.getSignInfoByPatient(getRepUID(), 0, 0, 15);
//			Page<Object> list = signWebService.getSignInfoByPatient(getUID(), 0, 0, 15);
            if (list.getTotalPages() > 0) {
                return error(-1, "已存在申请签约的医生,请确认!");
            }

            String errorMsg = "";
            if (StringUtils.isEmpty(doctor)) {
                errorMsg = "医生代码不能为空!";
            }

            if (StringUtils.isNotEmpty(errorMsg)) {
                return error(-1, errorMsg);
            }

            SignWeb signWeb = new SignWeb();
            signWeb.setUser(getRepUID());
//			signWeb.setUser(getUID());
            signWeb.setDoctor(doctor);
            signWeb.setStatus(0);
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            if (!StringUtils.isEmpty(startTime)) {
                Date beginDate = format.parse(startTime);
                signWeb.setBeginDate(beginDate);
            }
            if (!StringUtils.isEmpty(endTime)) {
                Date endDate = format.parse(endTime);
                signWeb.setEndDate(endDate);
            }
            // 操作人（1患者，2医生）
            signWeb.setOper(1);
            signWeb.setApplyDate(new Date());
            boolean result = signWebService.saveSignWeb(signWeb);
            if (!result) {
                return error(-1, getResultMsg(result, 0));
            }

            return success(getResultMsg(result, 0));
        } catch (Exception ex) {
            error(ex);
            return error(-1, "系统错误,请联系管理员!");
        }

    }

    /**
     * 修改家庭签约状态
     *
     * @param code   签约标识
     * @param status 签约状态(0待签约，1已拒绝，2已签约，3患者申请取消签约，4医生申请取消签约，5已解约)
     * @return
     */
    @RequestMapping("/update_status")
    public String updateSignWeb(@RequestParam(value = "code") String code,
                                @RequestParam(value = "status") int status,
                                @RequestParam(value = "startDate", required = false) String startDate,
                                @RequestParam(value = "endDate", required = false) String endDate) {
        try {
            SignWeb signWeb = signWebService.getSignWebByCode(code);
            if (signWeb == null) {
                return error(-1, "签约记录不存在!");
            }

            signWeb.setStatus(status);
            if (StringUtils.isNotEmpty(startDate)) {
                if (StringUtils.isEmpty(endDate)) {
                    return error(-1, "到期时间不能为空!");
                }

                SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
                Date beginDate = format.parse(startDate);
                signWeb.setBeginDate(beginDate);
                Date endDateTime = format.parse(endDate);
                signWeb.setEndDate(endDateTime);
            }
            signWeb.setApplyDate(new Date());
            boolean result = signWebService.saveSignWeb(signWeb);
            if (!result) {
                return error(-1, getResultMsg(result, status));
            }

            return success(getResultMsg(result, status));
        } catch (Exception ex) {
            error(ex);
            return error(-1, "系统错误,请联系管理员!");
        }
    }

    /**
     * 返回操作提醒
     *
     * @param result 操作结果
     * @param status 签约状态
     * @return
     */
    public String getResultMsg(boolean result, int status) {
        String msg = "申请成功!";
        switch (status) {
            case 0:
                if (!result) {
                    msg = "申请失败!";
                }
                break;
            case 1:
                msg = "已拒绝申请!";
                if (!result) {
                    msg = "拒绝失败!";
                }
                break;
            case 2:
                msg = "签约成功!";
                if (!result) {
                    msg = "签约失败!";
                }
                break;
            case 3:
            case 4:
                msg = "申请解约成功!";
                if (!result) {
                    msg = "申请解约失败!";
                }
                break;
            case 5:
                msg = "解约成功!";
                if (!result) {
                    msg = "解约失败!";
                }
                break;
            case 6:
                msg = "取消成功!";
                if (!result) {
                    msg = "取消失败!";
                }
                break;
        }

        return msg;
    }

    /**
     * 获取签约医生信息  状态未传查全部签约信息
     *
     * @param status 签约状态 (0待签约，1已拒绝，2已签约，3患者申请取消签约，4医生申请取消签约，5已解约)
     * @return
     */
    @RequestMapping("/sign_info")
    public String getSignInfo(@RequestParam(required = false) String status, @RequestParam(defaultValue = "0") long id, int pageSize) {
        try {
            int iStatus = 0;
            if (StringUtils.isEmpty(status)) {
                iStatus = -1;
            } else {
                iStatus = Integer.parseInt(status);
            }

//			Page<Object> listSign = signWebService.getSignInfoByPatient(getUID(), iStatus, id, pageSize);
            Page<Object> listSign = signWebService.getSignInfoByPatient(getRepUID(), iStatus, id, pageSize);

            JSONArray jsonArray = new JSONArray();
            if (listSign != null) {
                for (Object obj : listSign) {
                    Object[] temp = (Object[]) obj;
                    if (temp == null) {
                        continue;
                    }
                    JSONObject json = new JSONObject();
                    json.put("code", temp[0]);
                    json.put("patientCode", temp[1]);
                    json.put("doctorCode", temp[2]);
                    json.put("doctorName", temp[3]);
                    json.put("hosptialName", temp[4]);
                    json.put("deptName", temp[5]);
                    json.put("jobName", temp[6]);
                    iStatus = (Integer) temp[7];
                    json.put("status", iStatus);
                    json.put("photo", temp[8]);
                    switch (iStatus) {
                        case 0:
                            status = "待签约";
                            break;
                        case 1:
                            status = "已拒绝";
                            break;
                        case 2:
                            status = "已签约";
                            break;
                        case 3:
                        case 4:
                            status = "申请解约";
                            break;
                        case 5:
                            status = "已解约";
                            break;
                        case 6:
                            status = "已取消";
                            break;
                    }
                    json.put("statusName", status);
                    json.put("id", temp[9]);

                    jsonArray.put(json);
                }
            }

            return write(200, "查询成功！", "list", jsonArray);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "系统错误,请联系管理员!");
        }
    }

    @RequestMapping("/sign_doctor_info")
    public String getSignDoctorInfo(@RequestParam(required = true, value = "team") String teamCode,
                                    @RequestParam(required = true) String code) {
        try {
            JSONObject resultObj = signWebService.getSignInfoOrRenewInfo(teamCode, code);
            return write(200, "查询成功！", "data", resultObj);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "系统错误,请联系管理员!");
        }
    }


    @RequestMapping(value = "/consult_sign_doctor_info")
    @ResponseBody
    @ApiOperation(value = "获取咨询的服务团队")
    public String consult_sign_doctor(@RequestParam(required = true, value = "team") String teamCode,
                                      @RequestParam(required = true) String code,
                                      @RequestParam(required = false) String consult,
                                      @RequestParam(required = false) Integer status) {
        try {
            JSONObject resultObj = signWebService.getConsultSignInfoOrRenewInfo(teamCode, code, getRepUID(), consult, status);
            return write(200, "查询成功！", "data", resultObj);
        } catch (Exception e) {
            error(e);
            return error(-1, "签约状态查询失败！");
        }
    }

    @RequestMapping("/sign_service_info")
    public String getSignServiceInfo(@RequestParam(required = true) String signCode) {
        try {
            JSONObject resultObj = signWebService.getSignServiceInfo(signCode);
            return write(200, "查询成功！", "data", resultObj);
        } catch (Exception ex) {
            error(ex);
            return error(-1, "系统错误,请联系管理员!");
        }
    }

    @RequestMapping("/signs")
    public String getSigns() {
        try {
            List<JSONObject> jsonArray = signWebService.getSigns(getRepUID());
//			List<JSONObject> jsonArray  = signWebService.getSigns(getUID());
            //JSONArray jsonArray  = signWebService.getSigns("915cc456-5b1d-11e6-8344-fa163e8aee56");
            return write(200, "查询成功！", "data", new JSONArray(jsonArray));
        } catch (Exception ex) {
            error(ex);
            return error(-1, "系统错误,请联系管理员!");
        }
    }

    /**
     * 根据签约code查询签约详情
     * @param signCode
     * @return
     */
    @RequestMapping(value = "/signInfo",method = RequestMethod.GET)
    public String getSignInfo(@RequestParam String signCode) {
        try {
            List<JSONObject> jsonArray = signWebService.getSignInfo(signCode);
            return write(200, "查询成功！", "data", new JSONArray(jsonArray));
        } catch (Exception ex) {
            error(ex);
            return error(-1, "系统错误,请联系管理员!");
        }
    }

    /**
     * 获取转签原因字段
     *
     * @return
     */
    @RequestMapping(value = "/getRenewChangeReason", method = {RequestMethod.GET})
    @ResponseBody
    public String getRenewChangeReason() {
        try {
            JSONArray rs = signWebService.getRenewChangeReason();
            return write(200, "查询成功", "data", rs);
        } catch (Exception e) {
            return error(-1, "查询失败");
        }
    }

    /**
     * 获取厦门市区
     * @return
     */
    @RequestMapping(value = "/getTownList", method = {RequestMethod.GET, RequestMethod.POST})
    @ResponseBody
    public String getTownList(){
        try {
            return write(200, "查询成功", "data", signWebService.getTownList());
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败");
        }
    }

    /**
     * 获取街道下居委会
     * @param street
     * @return
     */
    @RequestMapping(value = "/getCountryListByStreet", method = {RequestMethod.GET, RequestMethod.POST})
    @ResponseBody
    public String getCountryListByStreet(@RequestParam(required = true)String street){
        try {
            return write(200, "查询成功", "data", signWebService.getCountryListByStreet(street));
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败");
        }
    }

    /**
     * 获取居委会字段
     * @param countryCode
     * @return
     */
    @RequestMapping(value = "/updatePatientCountry", method = {RequestMethod.GET, RequestMethod.POST})
    @ResponseBody
    public String setPatientCountry(@RequestParam(required = true)String countryCode){
        try {
            return write(200, "保存成功", "data", signWebService.setPatientCountry(countryCode,getRepUID()));
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败");
        }
    }

    /**
     * 获取区域下街道
     * @param town
     * @return
     */
    @RequestMapping(value = "/getStreetListByTown", method = {RequestMethod.GET, RequestMethod.POST})
    @ResponseBody
    public String getStreetListByTown(@RequestParam(required = true)String town){
        try {
            return write(200, "查询成功", "data", signWebService.getStreetListByTown(town));
        } catch (Exception e) {
            error(e);
            return error(-1, "查询失败");
        }

    }
}
